@@ -9,23 +9,23 @@ end
 const wakerust = Ref{Ptr{Cvoid}}(C_NULL)
 const droparray = Ref{Ptr{Cvoid}}(C_NULL)
 
-function runasync(func::Function, wakeptr::Ptr{Cvoid}, args...)::Any
+function runasync(func::Function, wakeptr::Ptr{Cvoid}, args...; kwargs...)::Any
     try
-        func(args...)
+        func(args...; kwargs...)
     finally
         ccall(wakerust[], Cvoid, (Ptr{Cvoid},), wakeptr)
     end
 end
 
-function asynccall(func::Function, wakeptr::Ptr{Cvoid}, args...)::Task
+function asynccall(func::Function, wakeptr::Ptr{Cvoid}, args...; kwargs...)::Task
     @assert wakerust[] != C_NULL "wakerust is null"
-    Base.Threads.@spawn runasync(func, wakeptr, args...)
+    Base.Threads.@spawn runasync(func, wakeptr, args...; kwargs...)
 end
 
 function tracingcall(func::Function)::Function
-    function wrapper(args...)
+    function wrapper(args...; kwargs...)
         try
-            func(args...)
+            func(args...; kwargs...)
         catch
             for s in stacktrace(catch_backtrace(), true)
                 println(stderr, s)
@@ -39,9 +39,9 @@ function tracingcall(func::Function)::Function
 end
 
 function attachstacktrace(func::Function)::Function
-    function wrapper(args...)
+    function wrapper(args...; kwargs...)
         try
-            func(args...)
+            func(args...; kwargs...)
         catch exc
             st::StackTrace = stacktrace(catch_backtrace(), true)
             rethrow(TracedException(exc, st))
@@ -55,4 +55,15 @@ function clean(a::Array)
     @assert droparray[] != C_NULL "droparray is null"
     ccall(droparray[], Cvoid, (Array,), a)
 end
+
+@nospecialize
+function displaystring(value)::String
+    try
+        io = IOBuffer()
+        display(TextDisplay(io), value)
+        String(take!(io))
+    catch
+        ""
+    end
+end
 end
