@@ -7,7 +7,7 @@ fn create_large_array() {
         let mut jlrs = j.borrow_mut();
 
         jlrs.scope_with_slots(1, |_, frame| {
-            let array = Value::new_array::<f32, _, _, _>(frame, &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
+            let array = Array::new::<f32, _, _, _>(frame, &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
             assert!(array.is_ok());
             Ok(())
         })
@@ -21,7 +21,7 @@ fn move_large_array() {
         let mut jlrs = j.borrow_mut();
 
         jlrs.scope_with_slots(1, |_, frame| {
-            let array = Value::move_array(frame, vec![1u64], &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
+            let array = Array::from_vec(frame, vec![1u64], &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
             assert!(array.is_ok());
             Ok(())
         })
@@ -36,7 +36,7 @@ fn borrow_large_array() {
 
         jlrs.scope_with_slots(1, |_, frame| {
             let mut data = vec![1u32];
-            let array = Value::borrow_array(frame, &mut data, &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
+            let array = Array::from_slice(frame, &mut data, &[1, 1, 1, 1, 1, 1, 1, 1, 1][..]);
             assert!(array.is_ok());
             Ok(())
         })
